from colour_printing.markers import Markers

default_info = "info"
default_warn = "warn"
default_success = "success"
default_error = "error"


class ColourPrint:
    def __init__(self):
        self.__default_setting()

    def new_flag(self, flag: str):
        setattr(self, flag, Markers(flag))
        if Markers.flag_len < len(flag):
            Markers.flag_len = len(flag)

    def __default_setting(self):
        info = Markers(default_info)
        info.set_flag_style(mode='bold', fore='blue')
        info.set_time_style(mode='invert')
        info.set_str_style(mode='bold', fore='blue')
        setattr(self, default_info, info)

        warn = Markers(default_warn)
        warn.set_flag_style(mode='bold', fore='yellow')
        warn.set_time_style(mode='invert')
        warn.set_str_style(mode='bold', fore='yellow')
        setattr(self, default_warn, warn)

        success = Markers(default_success)
        success.set_flag_style(mode='bold', fore='green')
        success.set_time_style(mode='invert')
        success.set_str_style(mode='bold', fore='green')
        setattr(self, default_success, success)

        error = Markers(default_error)
        error.set_flag_style(mode='bold', fore='red')
        error.set_time_style(mode='invert')
        error.set_str_style(mode='bold', fore='red')
        setattr(self, default_error, error)

    def __str__(self):
        return """{
            'fore':
                {  # 前景色
                    'black': 30,  # 黑色
                    'red': 31,  # 红色
                    'green': 32,  # 绿色
                    'yellow': 33,  # 黄色
                    'blue': 34,  # 蓝色
                    'purple': 35,  # 紫红色
                    'cyan': 36,  # 青蓝色
                    'white': 37,  # 白色
                },

            'back':
                {  # 背景
                    'black': 40,  # 黑色
                    'red': 41,  # 红色
                    'green': 42,  # 绿色
                    'yellow': 43,  # 黄色
                    'blue': 44,  # 蓝色
                    'purple': 45,  # 紫红色
                    'cyan': 46,  # 青蓝色
                    'white': 47,  # 白色
                },

            'mode':
                {  # 显示模式
                    'mormal': 0,  # 终端默认设置
                    'bold': 1,  # 高亮显示
                    'underline': 4,  # 使用下划线
                    'blink': 5,  # 闪烁
                    'invert': 7,  # 反白显示
                    'hide': 8,  # 不可见
                },

        }"""












from datetime import datetime
from colour_printing.style import STYLE, STR_STYLE, FLAG_STYLE, TIME_STYLE
from colour_printing.switch import Switch

class Markers:
    time = lambda _: datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')
    template = '{flag_style0}[{flag}]{flag_style1} {time_style0}{time}{time_style1} {str_style0}{string}{str_style1}'
    flag_len = 7

    def __init__(self, flag: str):
        self.flag = flag.upper()
        self.config = {}

    @classmethod
    def __setting(cls, mode='', fore='', back=''):
        """
        linxu:转义序列以ESC开头，即ASCII码下的\033
                   格式: \033[显示方式;前景色;背景色m
        """
        mode = '%s' % STYLE['mode'][mode] if STYLE['mode'].get(mode) else ''

        fore = '%s' % STYLE['fore'][fore] if STYLE['fore'].get(fore) else ''

        back = '%s' % STYLE['back'][back] if STYLE['back'].get(back) else ''

        style = ';'.join([s for s in [mode, fore, back] if s])

        style = '\033[%sm' % style if style else ''

        end = '\033[%sm' % STYLE['default']['end'] if style else ''

        return style, end

    def set_str_style(self, mode='', fore='', back=''):
        self.config[STR_STYLE] = self.__setting(mode=mode, fore=fore, back=back)

    def set_time_style(self, mode='', fore='', back=''):
        self.config[TIME_STYLE] = self.__setting(mode=mode, fore=fore, back=back)

    def set_flag_style(self, mode='', fore='', back=''):
        self.config[FLAG_STYLE] = self.__setting(mode=mode, fore=fore, back=back)

    def __user_setting(self):
        str_style = self.config.get(STR_STYLE, self.__setting())
        time_style = self.config.get(TIME_STYLE, self.__setting())
        flag_style = self.config.get(FLAG_STYLE, self.__setting())
        return self.template.format(flag_style0=flag_style[0],
                                    flag=self.flag.center(self.flag_len, '-'),
                                    flag_style1=flag_style[1],
                                    time_style0=time_style[0],
                                    time=self.time(),
                                    time_style1=time_style[1],
                                    str_style0=str_style[0],
                                    string='{}',
                                    str_style1=str_style[1])

    def __call__(self, *args):
        if not Switch.signal or self.flag.lower() in Switch.filter or \
                self.flag in Switch.filter:
            return
        template = self.__user_setting().replace('{}', len(args) * '{} ')
        print(template.format(*args))
